# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: renderserver.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="renderserver.proto",
    package="renderserver",
    syntax="proto3",
    serialized_options=None,
    serialized_pb=b'\n\x12renderserver.proto\x12\x0crenderserver"\x0e\n\x0c\x45mptyRequest"\x0f\n\rEmptyResponse"\x1f\n\x0fNewSceneRequest\x12\x0c\n\x04name\x18\x01 \x01(\t"\x12\n\x10NewSceneResponse"+\n\tAnimation\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64uration\x18\x02 \x01(\x02"\x18\n\x16\x41nimationStatusRequest"\x19\n\x17\x41nimationStatusResponse"m\n\x12ManimStatusRequest\x12\x12\n\nscene_name\x18\x01 \x01(\t\x12\x16\n\x0escene_finished\x18\x02 \x01(\x08\x12+\n\nanimations\x18\x03 \x03(\x0b\x32\x17.renderserver.Animation"\x15\n\x13ManimStatusResponse"\x14\n\x12UpdateSceneRequest"\x15\n\x13UpdateSceneResponse2\x94\x03\n\x0cRenderServer\x12J\n\x0f\x41nimationStatus\x12\x1a.renderserver.EmptyRequest\x1a\x1b.renderserver.EmptyResponse\x12R\n\x0bManimStatus\x12 .renderserver.ManimStatusRequest\x1a!.renderserver.ManimStatusResponse\x12R\n\x0bUpdateScene\x12 .renderserver.UpdateSceneRequest\x1a!.renderserver.UpdateSceneResponse\x12\x46\n\x08NewScene\x12\x1d.renderserver.NewSceneRequest\x1a\x1b.renderserver.EmptyResponse\x12H\n\rSceneFinished\x12\x1a.renderserver.EmptyRequest\x1a\x1b.renderserver.EmptyResponseb\x06proto3',
)


_EMPTYREQUEST = _descriptor.Descriptor(
    name="EmptyRequest",
    full_name="renderserver.EmptyRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=36,
    serialized_end=50,
)


_EMPTYRESPONSE = _descriptor.Descriptor(
    name="EmptyResponse",
    full_name="renderserver.EmptyResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=52,
    serialized_end=67,
)


_NEWSCENEREQUEST = _descriptor.Descriptor(
    name="NewSceneRequest",
    full_name="renderserver.NewSceneRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="renderserver.NewSceneRequest.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=69,
    serialized_end=100,
)


_NEWSCENERESPONSE = _descriptor.Descriptor(
    name="NewSceneResponse",
    full_name="renderserver.NewSceneResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=102,
    serialized_end=120,
)


_ANIMATION = _descriptor.Descriptor(
    name="Animation",
    full_name="renderserver.Animation",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="renderserver.Animation.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="duration",
            full_name="renderserver.Animation.duration",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=122,
    serialized_end=165,
)


_ANIMATIONSTATUSREQUEST = _descriptor.Descriptor(
    name="AnimationStatusRequest",
    full_name="renderserver.AnimationStatusRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=167,
    serialized_end=191,
)


_ANIMATIONSTATUSRESPONSE = _descriptor.Descriptor(
    name="AnimationStatusResponse",
    full_name="renderserver.AnimationStatusResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=193,
    serialized_end=218,
)


_MANIMSTATUSREQUEST = _descriptor.Descriptor(
    name="ManimStatusRequest",
    full_name="renderserver.ManimStatusRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="scene_name",
            full_name="renderserver.ManimStatusRequest.scene_name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="scene_finished",
            full_name="renderserver.ManimStatusRequest.scene_finished",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="animations",
            full_name="renderserver.ManimStatusRequest.animations",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=220,
    serialized_end=329,
)


_MANIMSTATUSRESPONSE = _descriptor.Descriptor(
    name="ManimStatusResponse",
    full_name="renderserver.ManimStatusResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=331,
    serialized_end=352,
)


_UPDATESCENEREQUEST = _descriptor.Descriptor(
    name="UpdateSceneRequest",
    full_name="renderserver.UpdateSceneRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=354,
    serialized_end=374,
)


_UPDATESCENERESPONSE = _descriptor.Descriptor(
    name="UpdateSceneResponse",
    full_name="renderserver.UpdateSceneResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=376,
    serialized_end=397,
)

_MANIMSTATUSREQUEST.fields_by_name["animations"].message_type = _ANIMATION
DESCRIPTOR.message_types_by_name["EmptyRequest"] = _EMPTYREQUEST
DESCRIPTOR.message_types_by_name["EmptyResponse"] = _EMPTYRESPONSE
DESCRIPTOR.message_types_by_name["NewSceneRequest"] = _NEWSCENEREQUEST
DESCRIPTOR.message_types_by_name["NewSceneResponse"] = _NEWSCENERESPONSE
DESCRIPTOR.message_types_by_name["Animation"] = _ANIMATION
DESCRIPTOR.message_types_by_name["AnimationStatusRequest"] = _ANIMATIONSTATUSREQUEST
DESCRIPTOR.message_types_by_name["AnimationStatusResponse"] = _ANIMATIONSTATUSRESPONSE
DESCRIPTOR.message_types_by_name["ManimStatusRequest"] = _MANIMSTATUSREQUEST
DESCRIPTOR.message_types_by_name["ManimStatusResponse"] = _MANIMSTATUSRESPONSE
DESCRIPTOR.message_types_by_name["UpdateSceneRequest"] = _UPDATESCENEREQUEST
DESCRIPTOR.message_types_by_name["UpdateSceneResponse"] = _UPDATESCENERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EmptyRequest = _reflection.GeneratedProtocolMessageType(
    "EmptyRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _EMPTYREQUEST,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.EmptyRequest)
    },
)
_sym_db.RegisterMessage(EmptyRequest)

EmptyResponse = _reflection.GeneratedProtocolMessageType(
    "EmptyResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _EMPTYRESPONSE,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.EmptyResponse)
    },
)
_sym_db.RegisterMessage(EmptyResponse)

NewSceneRequest = _reflection.GeneratedProtocolMessageType(
    "NewSceneRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _NEWSCENEREQUEST,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.NewSceneRequest)
    },
)
_sym_db.RegisterMessage(NewSceneRequest)

NewSceneResponse = _reflection.GeneratedProtocolMessageType(
    "NewSceneResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _NEWSCENERESPONSE,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.NewSceneResponse)
    },
)
_sym_db.RegisterMessage(NewSceneResponse)

Animation = _reflection.GeneratedProtocolMessageType(
    "Animation",
    (_message.Message,),
    {
        "DESCRIPTOR": _ANIMATION,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.Animation)
    },
)
_sym_db.RegisterMessage(Animation)

AnimationStatusRequest = _reflection.GeneratedProtocolMessageType(
    "AnimationStatusRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _ANIMATIONSTATUSREQUEST,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.AnimationStatusRequest)
    },
)
_sym_db.RegisterMessage(AnimationStatusRequest)

AnimationStatusResponse = _reflection.GeneratedProtocolMessageType(
    "AnimationStatusResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _ANIMATIONSTATUSRESPONSE,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.AnimationStatusResponse)
    },
)
_sym_db.RegisterMessage(AnimationStatusResponse)

ManimStatusRequest = _reflection.GeneratedProtocolMessageType(
    "ManimStatusRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _MANIMSTATUSREQUEST,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.ManimStatusRequest)
    },
)
_sym_db.RegisterMessage(ManimStatusRequest)

ManimStatusResponse = _reflection.GeneratedProtocolMessageType(
    "ManimStatusResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _MANIMSTATUSRESPONSE,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.ManimStatusResponse)
    },
)
_sym_db.RegisterMessage(ManimStatusResponse)

UpdateSceneRequest = _reflection.GeneratedProtocolMessageType(
    "UpdateSceneRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _UPDATESCENEREQUEST,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.UpdateSceneRequest)
    },
)
_sym_db.RegisterMessage(UpdateSceneRequest)

UpdateSceneResponse = _reflection.GeneratedProtocolMessageType(
    "UpdateSceneResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _UPDATESCENERESPONSE,
        "__module__": "renderserver_pb2"
        # @@protoc_insertion_point(class_scope:renderserver.UpdateSceneResponse)
    },
)
_sym_db.RegisterMessage(UpdateSceneResponse)


_RENDERSERVER = _descriptor.ServiceDescriptor(
    name="RenderServer",
    full_name="renderserver.RenderServer",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    serialized_start=400,
    serialized_end=804,
    methods=[
        _descriptor.MethodDescriptor(
            name="AnimationStatus",
            full_name="renderserver.RenderServer.AnimationStatus",
            index=0,
            containing_service=None,
            input_type=_EMPTYREQUEST,
            output_type=_EMPTYRESPONSE,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name="ManimStatus",
            full_name="renderserver.RenderServer.ManimStatus",
            index=1,
            containing_service=None,
            input_type=_MANIMSTATUSREQUEST,
            output_type=_MANIMSTATUSRESPONSE,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name="UpdateScene",
            full_name="renderserver.RenderServer.UpdateScene",
            index=2,
            containing_service=None,
            input_type=_UPDATESCENEREQUEST,
            output_type=_UPDATESCENERESPONSE,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name="NewScene",
            full_name="renderserver.RenderServer.NewScene",
            index=3,
            containing_service=None,
            input_type=_NEWSCENEREQUEST,
            output_type=_EMPTYRESPONSE,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name="SceneFinished",
            full_name="renderserver.RenderServer.SceneFinished",
            index=4,
            containing_service=None,
            input_type=_EMPTYREQUEST,
            output_type=_EMPTYRESPONSE,
            serialized_options=None,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_RENDERSERVER)

DESCRIPTOR.services_by_name["RenderServer"] = _RENDERSERVER

# @@protoc_insertion_point(module_scope)
